@@ -40,8 +40,8 @@ class NSLClient:
     @property
     def default_nsl_id(self):
         """
-        if you don't set the nsl_id for each stac request, then this nsl_id is the default choice. if you set this
-        default value you must make sure that the nsl_id has already been 'set' by calling `set_credentials`
+        if you don't set the nsl_id for each stac request, then this nsl_id is the default choice.
+        if you set this default value you must make sure that the nsl_id has already been 'set' by calling `set_credentials`
         :return:
         """
         return bearer_auth.default_nsl_id
@@ -148,18 +148,23 @@ class NSLClient:
                 else:
                     yield item
         else:
-            page_size = 100
-            count = 0
             limit = stac_request.limit if stac_request.limit > 0 else None
             offset = stac_request.offset
+            page_size = 500
+            count = 0
 
             stac_request.limit = page_size
             items = list(self.search(stac_request, timeout=timeout, nsl_id=nsl_id, profile_name=profile_name))
             while len(items) > 0:
                 for item in items:
-                    if limit is None or count < limit:
-                        count += 1
+                    if limit is None or (limit is not None and count < limit):
                         yield item
+                        count += 1
+                    if limit is not None and count >= limit:
+                        break
+
+                if limit is not None and count >= limit:
+                    break
 
                 stac_request.offset += page_size
                 items = list(self.search(stac_request, timeout=timeout, nsl_id=nsl_id, profile_name=profile_name))
