@@ -300,14 +300,16 @@ does the AssetWrap equal a protobuf Asset
                  save_filename: str = '',
                  save_directory: str = '',
                  requester_pays: bool = False,
-                 nsl_id: str = None) -> str:
+                 nsl_id: str = None,
+                 profile_name: str = None) -> str:
         return utils.download_asset(asset=self._asset,
                                     from_bucket=from_bucket,
                                     file_obj=file_obj,
                                     save_filename=save_filename,
                                     save_directory=save_directory,
                                     requester_pays=requester_pays,
-                                    nsl_id=nsl_id)
+                                    nsl_id=nsl_id,
+                                    profile_name=profile_name)
 
     def matches_details(self,
                         asset_key: str = None,
@@ -814,7 +816,9 @@ then that supersedes this projection definition.
                        from_bucket: bool = False,
                        file_obj: BinaryIO = None,
                        save_filename: str = "",
-                       save_directory: str = "") -> str:
+                       save_directory: str = "",
+                       nsl_id: str = None,
+                       profile_name: str = None) -> str:
         asset_wrap = self.get_asset(asset_key=asset_key,
                                     asset_type=asset_type,
                                     cloud_platform=cloud_platform,
@@ -824,7 +828,9 @@ then that supersedes this projection definition.
         return asset_wrap.download(from_bucket=from_bucket,
                                    file_obj=file_obj,
                                    save_filename=save_filename,
-                                   save_directory=save_directory)
+                                   save_directory=save_directory,
+                                   nsl_id=nsl_id,
+                                   profile_name=profile_name)
 
     def equals_pb(self, other: StacItem):
         """
@@ -1091,7 +1097,7 @@ other quad STAC items that are contained by '02313012030' are returned.
 
     def set_azimuth(self,
                     rel_type: enum.FilterRelationship,
-                    value: float,
+                    value: float = None,
                     start: float = None,
                     end: float = None,
                     sort_direction: enum.SortDirection = enum.SortDirection.NOT_SORTED):
@@ -1111,7 +1117,7 @@ other quad STAC items that are contained by '02313012030' are returned.
 
     def set_off_nadir(self,
                       rel_type: enum.FilterRelationship,
-                      value: float,
+                      value: float = None,
                       start: float = None,
                       end: float = None,
                       sort_direction: enum.SortDirection = enum.SortDirection.NOT_SORTED):
@@ -1123,7 +1129,7 @@ other quad STAC items that are contained by '02313012030' are returned.
 
     def set_sun_azimuth(self,
                         rel_type: enum.FilterRelationship,
-                        value: float,
+                        value: float = None,
                         start: float = None,
                         end: float = None,
                         sort_direction: enum.SortDirection = enum.SortDirection.NOT_SORTED):
@@ -1135,7 +1141,7 @@ other quad STAC items that are contained by '02313012030' are returned.
 
     def set_sun_elevation(self,
                           rel_type: enum.FilterRelationship,
-                          value: float,
+                          value: float = None,
                           start: float = None,
                           end: float = None,
                           sort_direction: enum.SortDirection = enum.SortDirection.NOT_SORTED):
@@ -1159,7 +1165,7 @@ other quad STAC items that are contained by '02313012030' are returned.
 
     def set_gsd(self,
                 rel_type: enum.FilterRelationship,
-                value: float,
+                value: float = None,
                 start: float = None,
                 end: float = None,
                 sort_direction: enum.SortDirection = enum.SortDirection.NOT_SORTED):
